#include <string.h>
#define GC_THREADS

#ifdef __cplusplus
# include "gc_cpp.h"
#else
# include "gc.h"
# define malloc(n) GC_MALLOC(n)
# define calloc(m,n) GC_MALLOC((m)*(n))
# define free(p) GC_FREE(p)
# define realloc(p,n) GC_REALLOC((p),(n))
#endif

extern void xalloc_die (void) __attribute__ ((__noreturn__));

static inline char *
my_strdup (char const *s)
{
  size_t len = strlen (s);
  void *t = GC_MALLOC (len + 1);
  if (t == NULL)
    return NULL;
  return (char *) memcpy (t, s, len + 1);
}
# undef strdup
# define strdup(s) my_strdup(s)

static inline char *
my_strndup (char const *s, size_t n)
{
  size_t len = strnlen (s, n);
  char *t = (char *) GC_MALLOC (len + 1);
  if (t == NULL)
    return NULL;
  t[len] = '\0';
  return (char *) memcpy (t, s, len);
}
# undef strndup
# define strndup(s, n) my_strndup(s, n)

static inline void *
xmalloc (size_t n)
{
  void *p = GC_MALLOC (n);
  if (!p && n != 0)
    xalloc_die ();
  return p;
}

static inline char *
xstrndup (const char *string, size_t n)
{
  char *s = strndup (string, n);
  if (! s)
    xalloc_die ();
  return s;
}

static inline void *
xmemdup (void const *p, size_t s)
{
  return memcpy (xmalloc (s), p, s);
}

static inline char *
xstrdup (char const *string)
{
  return (char *) xmemdup (string, strlen (string) + 1);
}
